/*
 * Copyright (C) 2012-2013 jeanfi@gmail.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>

#include <json/json.h>
#include <webkit/webkit.h>
#include <glib/gi18n.h>
#include <gtk/gtk.h>

#include "http.h"
#include "ttrss.h"
#include "webbrowser.h"

static const char *program_name;
static GSettings *settings;

static int model_state;

enum {
	COL_FEED_TITLE,
	COL_FEED_ID
};

enum {
	COL_HEADLINE_TITLE,
	COL_HEADLINE_ID
};

static struct option long_options[] = {
	{"version", no_argument, 0, 'v'},
	{"help", no_argument, 0, 'h'},
	{0, 0, 0, 0}
};

static void print_version()
{
	printf("prss %s\n", VERSION);
	printf(_("Copyright (C) %s jeanfi@gmail.com\n"
		 "License GPLv2: GNU GPL version 2 or later "
		 "<http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>\n"
		 "This is free software: you are free to change and "
		 " redistribute it.\n"
		 "There is NO WARRANTY, to the extent permitted by law.\n"),
	       "2012-2013");
}

static void print_help()
{
	printf(_("Usage: %s [OPTION]...\n"), program_name);

	puts(_("Prss is a GTK+ news reader for tt-rss."));

	puts("");
	puts(_("Options:"));
	puts(_("  -h, --help          display this help and exit\n"
	       "  -v, --version       display version information and exit"));

	puts("");

	printf(_("Report bugs to: %s\n"), PACKAGE_BUGREPORT);
	puts("");
	printf(_("%s home page: <%s>\n"), PACKAGE_NAME, PACKAGE_URL);
}

GtkTreeView *w_feedtree;
GtkTreeView *w_headlinetree;

void update()
{
	struct feed **feeds;
	GtkTreeModel *model, *headlinemodel;
	GtkTreeIter iter;
	char *title;

	model = gtk_tree_view_get_model(GTK_TREE_VIEW(w_feedtree));

	printf("update(): clear feed tree\n");
	model_state = 1;
	gtk_list_store_clear(GTK_LIST_STORE(model));
	headlinemodel = gtk_tree_view_get_model(w_headlinetree);
	gtk_list_store_clear(GTK_LIST_STORE(headlinemodel));
	printf("update(): clear feed tree done\n");
	feeds = ttrss_get_feeds();
	while (feeds && *feeds) {
		if ((*feeds)->unread)
			title = g_strdup_printf("<b>%s (%d)</b>",
						(*feeds)->title,
						(*feeds)->unread);
		else
			title = strdup((*feeds)->title);


		gtk_list_store_append(GTK_LIST_STORE(model), &iter);

		gtk_list_store_set(GTK_LIST_STORE(model),
				   &iter,
				   0, title,
				   1, (*feeds)->id,
				   -1);
		free(title);
		feeds++;
	}
	model_state = 0;

	printf("refresh done\n");
}

void refresh_clicked_cbk(GtkWidget *btn, gpointer data)
{
	printf("refresh_clicked_cbk\n");
	update();
}

void settings_activate_cbk(GtkWidget *menu_item, gpointer data)
{
	GtkDialog *diag;
	GtkBuilder *builder;
	GtkEntry *login, *url, *pwd;

	printf("settings_activate_cbk\n");

	builder = gtk_builder_new();

	gtk_builder_add_from_file
		(builder,
		 PACKAGE_DATA_DIR G_DIR_SEPARATOR_S "prss.glade",
		 NULL);

	diag = GTK_DIALOG(gtk_builder_get_object(builder, "settings_dialog"));

	url = GTK_ENTRY(gtk_builder_get_object(builder, "url_entry"));
	gtk_entry_set_text(url, g_settings_get_string(settings, "url"));

	login = GTK_ENTRY(gtk_builder_get_object(builder, "login_entry"));
	gtk_entry_set_text(login, g_settings_get_string(settings, "user"));

	pwd = GTK_ENTRY(gtk_builder_get_object(builder, "password_entry"));
	gtk_entry_set_text(pwd, g_settings_get_string(settings, "password"));

	if (gtk_dialog_run(diag)) {
		g_settings_set_string(settings,
				      "password",
				      gtk_entry_get_text(pwd));
		g_settings_set_string(settings,
				      "url",
				      gtk_entry_get_text(url));
		g_settings_set_string(settings,
				      "user",
				      gtk_entry_get_text(login));

		update();
	}

	gtk_widget_destroy(GTK_WIDGET(diag));

	g_object_unref(G_OBJECT(builder));
}

static struct feed *get_selected_feed()
{
	GtkTreePath *path;
	GtkTreeViewColumn *cols;
	GtkTreeIter iter;
	GtkTreeModel *model;
	int id;

	gtk_tree_view_get_cursor(w_feedtree, &path, &cols);

	if (path) {
		model = gtk_tree_view_get_model(w_feedtree);
		gtk_tree_model_get_iter(model, &iter, path);
		gtk_tree_model_get(model, &iter, 1, &id, -1);

		gtk_tree_path_free(path);

		return ttrss_get_feed(id);
	}

	return NULL;
}

static struct headline *get_selected_headline(GtkTreeIter *iter)
{
	GtkTreePath *path;
	GtkTreeViewColumn *cols;
	GtkTreeModel *model;
	int id;

	gtk_tree_view_get_cursor(w_headlinetree, &path, &cols);

	if (path) {
		model = gtk_tree_view_get_model(w_headlinetree);
		gtk_tree_model_get_iter(model, iter, path);
		gtk_tree_model_get(model, iter, 1, &id, -1);

		gtk_tree_path_free(path);

		return ttrss_get_headline(id);
	}

	return NULL;
}

int feed_cursor_changed_cbk(GtkTreeView *treeview, gpointer data)
{
	GtkTreeIter iter;
	GtkTreeModel *headline_model;
	GtkListStore *headline_store;
	struct feed *feed;
	struct headline **headlines;
	char *title;

	if (model_state)
		return TRUE;

	printf("feed_cursor_changed_cbk\n");

	feed = get_selected_feed();

	if (feed) {
		headline_model = gtk_tree_view_get_model(w_headlinetree);
		headline_store = GTK_LIST_STORE(headline_model);
		model_state = 1;
		gtk_list_store_clear(headline_store);

		if (feed) {
			headlines = ttrss_feed_get_headlines(feed);
			while (headlines && *headlines) {
				gtk_list_store_append(headline_store, &iter);

				if ((*headlines)->unread)
					title = g_strdup_printf
						("<b>%s</b>",
						 (*headlines)->title);
				else
					title = strdup((*headlines)->title);

				gtk_list_store_set(headline_store,
						   &iter,
						   0, title,
						   1, (*headlines)->id,
						   -1);

				free(title);

				headlines++;
			}
		}

		model_state = 0;
	}

	return 1;
}

int headline_cursor_changed_cbk(GtkTreeView *treeview, gpointer data)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	struct headline *headline;
	char *str;

	printf("headline_cursor_changed_cbk\n");

	if (model_state)
		return TRUE;

	headline = get_selected_headline(&iter);

	if (headline) {
		str = ttrss_get_headline_content(headline);

		if (str)
			web_load(str);
		else
			web_load("");

		free(str);

		if (headline->unread) {
			ttrss_set_article_unread(headline->id, 0);
			headline->unread = 0;
		}

		model = gtk_tree_view_get_model(w_headlinetree);
		gtk_list_store_set(GTK_LIST_STORE(model),
				   &iter,
				   0, headline->title,
				   -1);
	} else {
		printf("headline not found\n");
	}

	return TRUE;
}

void headline_row_activated_cbk(GtkWidget *widget, gpointer data)
{
	GdkScreen *screen;
	GError *error;
	GtkTreeSelection *selection;
	GtkTreeModel *model;
	GtkTreeIter iter;
	struct headline *h;
	const char *url;
	int id;

	printf("headline_row_activated_cbk\n");

	selection = gtk_tree_view_get_selection(w_headlinetree);
	model = gtk_tree_view_get_model(w_headlinetree);

	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, 1, &id, -1);

		h = ttrss_get_headline(id);
		if (h) {
			url = h->url;

			if (gtk_widget_has_screen(widget))
				screen = gtk_widget_get_screen(widget);
			else
				screen = gdk_screen_get_default();

			error = NULL;
			gtk_show_uri(screen, url,
				     gtk_get_current_event_time(), &error);
		}
	}
}

gboolean
window_delete_event_cbk(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	printf("window_delete_event_cbk\n");

	gtk_main_quit();

	return TRUE;
}

int main(int argc, char **argv)
{
	GtkWidget *window, *panh;
	GtkBuilder *builder;
	int optc, cmdok, opti;

	program_name = argv[0];

	setlocale(LC_ALL, "");

#if ENABLE_NLS
	bindtextdomain(PACKAGE, LOCALEDIR);
	textdomain(PACKAGE);
#endif

	cmdok = 1;
	while ((optc = getopt_long(argc, argv, "vh", long_options,
				   &opti)) != -1) {
		switch (optc) {
		case 'h':
			print_help();
			exit(EXIT_SUCCESS);
		case 'v':
			print_version();
			exit(EXIT_SUCCESS);
		default:
			cmdok = 0;
			break;
		}
	}

	if (!cmdok || optind != argc) {
		fprintf(stderr, _("Try `%s --help' for more information.\n"),
			program_name);
		exit(EXIT_FAILURE);
	}

	http_init();

	gtk_init(NULL, NULL);
	settings = g_settings_new("prss");

	ttrss_set_config(g_settings_get_string(settings, "url"),
			 g_settings_get_string(settings, "user"),
			 g_settings_get_string(settings, "password"));
	ws_open_session();

	builder = gtk_builder_new();
	gtk_builder_add_from_file
		(builder,
		 PACKAGE_DATA_DIR G_DIR_SEPARATOR_S "prss.glade",
		 NULL);
	window = GTK_WIDGET(gtk_builder_get_object(builder, "window"));
	w_feedtree = GTK_TREE_VIEW(gtk_builder_get_object(builder,
							  "feed_tree"));
	w_headlinetree = GTK_TREE_VIEW(gtk_builder_get_object(builder,
							      "headline_tree"));

	gtk_builder_connect_signals(builder, NULL);

	panh = GTK_WIDGET(gtk_builder_get_object(builder, "webkit_window"));

	gtk_container_add(GTK_CONTAINER(panh), web_get_widget());

	g_object_unref(G_OBJECT(builder));

	gtk_widget_show_all(window);

	gtk_main();

	exit(EXIT_SUCCESS);
}
